//
// SliderEntry.cpp
// Implementation for TSliderEntry class
//

#include <stdio.h>
#include <string>
#include "SliderEntry.h"

//
// Property Editors for slider properties
//
class THashMarksProperty: public TPickListProperty {
public:
	THashMarksProperty(TComponent *component, TProperty *prop)
	: TPickListProperty(component, prop)
	{}
	
	virtual void GetValuePickList(TStringList *list)
	{
		list->MakeEmpty();
		list->Add("B_HASH_MARKS_NONE");
		list->Add("B_HASH_MARKS_TOP");
		list->Add("B_HASH_MARKS_BOTTOM");
		list->Add("B_HASH_MARKS_BOTH");
	}
};

class TThumbStyleProperty: public TPickListProperty {
public:
	TThumbStyleProperty(TComponent *component, TProperty *prop)
	: TPickListProperty(component, prop)
	{}
	
	virtual void GetValuePickList(TStringList *list)
	{
		list->MakeEmpty();
		list->Add("B_BLOCK_THUMB");
		list->Add("B_TRIANGLE_THUMB");
	}
};


//
// TSliderEntry implementation
//
TComponent *TSliderEntry::CreateComponent(BView *view, TFormDesigner *designer, BRect rect)
{
	TDesignSlider *btn = new TDesignSlider(rect, "Slider", "Slider", designer);
	btn->SetClassName(ClassName());
	
	// default view properties
	btn->SetStyle(B_BLOCK_THUMB);
	btn->SetHashMarks(B_HASH_MARKS_BOTH);
	rgb_color clr = {0,128,255,0};
	btn->SetBarColor(clr);
	btn->SetHashMarkCount(10);
	btn->SetLimitLabels("Min", "Max");
	
	view->LockLooper();
	view->AddChild(btn);
	view->UnlockLooper();
	return btn;
}

//
// TDesignSlider implementation
//
TDesignSlider::TDesignSlider(BRect frame, const char *name, const char *caption, TFormDesigner *designer)
:	BSlider(frame, name, NULL/*caption*/, 0/*new BMessage('yeah')*/, 0, 10, B_BLOCK_THUMB)
, TControlComponent(0, designer, ctVisual)
{
	SetComponentName(name);
	SetDTView(this); // ALWAYS do this
	InitPropertyList(); // and this
	InitPropertyValues();
	InitEventList();
	InitEventValues();
}

void TDesignSlider::MouseDown(BPoint pt)
{
	DesignerMouseDown(pt, Bounds(), this);
	Invalidate();
}

void TDesignSlider::Draw(BRect updateRect)
{
	BSlider::Draw(updateRect);
	DesignerDrawSelection();
}

bool TDesignSlider::AddComponentContextMenuItems(BPopUpMenu *menu)
{
	return true;
}

bool TDesignSlider::HandleComponentContextMessage(BMessage *message)
{
	return false;
}

bool TDesignSlider::DefaultEditAction(BPoint /*pt*/)
{
	return false;
}

const char *TDesignSlider::GetPropertyValue(const char *propName) const
{
	string s = propName;
	
	return TControlComponent::GetPropertyValue(propName);
}

void TDesignSlider::SetPropertyValue(const char *propName, const char *value)
{
	printf("TDesignSlider::SetPropertyValue('%s', '%s')\n", propName, value);
	if((!propName) || (!value)) return;
	
	string s = propName;
	string v = value;
	
	if(s == "BarColor")
	{
		rgb_color clr;
		StrToRGBColor(v.c_str(), &clr);
		LockLooper();
		SetBarColor(clr);
		Invalidate();
		UnlockLooper();
		StoreProperty(propName, value);
		return;
	}
	
	if(s == "HashMarkPos")
	{
		LockLooper();
		
		if(v == "B_HASH_MARKS_NONE")
		{
			SetHashMarks(B_HASH_MARKS_NONE);
		}
		else if(v == "B_HASH_MARKS_TOP")
		{
			SetHashMarks(B_HASH_MARKS_TOP);
		}
		else if(v == "B_HASH_MARKS_BOTTOM")
		{
			SetHashMarks(B_HASH_MARKS_BOTTOM);
		}
		else if(v == "B_HASH_MARKS_BOTH")
		{
			SetHashMarks(B_HASH_MARKS_BOTH);
		}
		UnlockLooper();
		StoreProperty(propName, value);
		return;
	}
	
	if(s == "HashMarkCount")
	{
		int32 l = atol(v.c_str());
		LockLooper();
		SetHashMarkCount(l);
		UnlockLooper();
		StoreProperty(propName, value);
		return;
	}	
	
	if(s == "Style")
	{
		LockLooper();
		if(v == "B_TRIANGLE_THUMB")
			SetStyle(B_TRIANGLE_THUMB);
		else
			SetStyle(B_BLOCK_THUMB);
		UnlockLooper();
		StoreProperty(propName, value);
		return;
	}
	
	if(s == "MinValueLabel")
	{
		char buf[80] = "";
		LockLooper();
		strcpy(buf, MaxLimitLabel());
		SetLimitLabels(v.c_str(), buf);
		UnlockLooper();
		StoreProperty(propName, value);
		return;
	}
	
	if(s == "MaxValueLabel")
	{
		char buf[80] = "";
		LockLooper();
		strcpy(buf, MinLimitLabel());
		SetLimitLabels(buf, v.c_str());
		UnlockLooper();
		StoreProperty(propName, value);
		return;
	}
	
	if(s == "Value")
	{
		int32 l = atol(v.c_str());
		LockLooper();
		SetValue(l);
		UnlockLooper();
		StoreProperty(propName, value);
	}

	TControlComponent::SetPropertyValue(propName, value);
}

void TDesignSlider::InitPropertyList(void)
{
	TControlComponent::InitPropertyList();
	
	const char *csBarColorSet = 
		"{rgb_color clr = $VALUE;\n"
		"$COMPONENT->SetBarColor(clr);}"
		;

	AddPropAndEditor(
		new TProperty(this, "BarColor", "{0,128,255,0}")
	, new TRGBColorProperty(this, 0)
	, csBarColorSet
	);
	
	AddPropAndEditor(
		new TProperty(this, "HashMarks", "B_HASH_MARKS_BOTH")
	, new THashMarksProperty(this, 0)
	);
	
	AddPropAndEditor(
		new TProperty(this, "HashMarkCount", "10")
	, new TIntProperty(this, 0)
	);

	AddPropAndEditor(
		new TProperty(this, "Style", "B_BLOCK_THUMB")
	, new TThumbStyleProperty(this, 0)
	);

	AddPropAndEditor(
		new TProperty(this, "MinValue", "0")
	, new TIntProperty(this, 0)
	);

	AddPropAndEditor(
		new TProperty(this, "MaxValue", "10")
	, new TIntProperty(this, 0)
	);

	AddPropAndEditor(
		new TProperty(this, "MinValueLabel", "Min")
	, new TStringProperty(this, 0)
	, 0,0,true
	);

	AddPropAndEditor(
		new TProperty(this, "MaxValueLabel", "Max")
	, new TStringProperty(this, 0)
	, 0,0,true
	);
	
	AddPropAndEditor(
		new TProperty(this, "Value", "0")
	, new TIntProperty(this, 0)
	);
	

	// These are(only) set in the constructor
	FindProperty("MaxValue")->SetGenerated(false);
	FindProperty("MinValue")->SetGenerated(false);
}

void TDesignSlider::InitEventList(void)
{
	TControlComponent::InitEventList();
	
	AddEvent(new TEvent("OnDrawBar"));
	AddEvent(new TEvent("OnDrawFocusMark"));
	AddEvent(new TEvent("OnDrawHashMarks"));
	AddEvent(new TEvent("OnDrawSlider"));
	AddEvent(new TEvent("OnDrawText"));
	AddEvent(new TEvent("OnDrawThumb"));
	AddEvent(new TEvent("OnSliderChange"));
}

void TDesignSlider::InitPropertyValues(void)
{
	TControlComponent::InitPropertyValues();
}

void TDesignSlider::InitEventValues(void)
{
	TControlComponent::InitEventValues();
}

void TDesignSlider::OnSetConstructorParams(BString &s)
{
	TControlComponent::OnSetConstructorParams(s);	
	
	char buf[256] = "";
	sprintf(
		buf
	, "\"%s\", 0, %s, %s"
	, FindProperty("Label")->Value()
	, FindProperty("MinValue")->Value()
	, FindProperty("MaxValue")->Value()
	);
	s += buf;
}

bool TDesignSlider::OnGenInitForm(BString &s)
{
	char buf[200] = "";
	sprintf(
		buf
	, "$COMPONENT->SetLimitLabels(\"%s\", \"%s\");\n"
	, FindProperty("MinValueLabel")->Value()
	, FindProperty("MaxValueLabel")->Value()
	);
	
	s	<< buf;
	return true;
}

// eof
